indexing
	description: "[
	  " The instant messenger application -- use it to communicate with
		your friends over the internet!
	  ";
	  
	  Authors: Keno Albrecht and Angela Nicoara
	]"

	date: "$Date: 2005/04/14 14:42:40 $";
	revision: "$Revision: 1.3 $"
	
class
	INSTANT_MESSENGER

create
	make

feature

	make (argv: ARRAY [STRING]) is
		local
			server_address: STRING
			name: STRING
			port: INTEGER
			
			soc: NETWORK_DATAGRAM_SOCKET
			ps_write: MEDIUM_POLLER
			writecomm: MESSAGE_SENDER
			receiver_thread: RECEIVER_THREAD
			registration_thread: REGISTRATION_THREAD
			uiMenu: UIMENU
			
			user_list: LINKED_LIST[USER]
		do			
			if argv.count /= 4 then
				io.error.putstring ("Usage: ")
				io.error.putstring (argv.item (0))
				io.error.putstring (" hostname username portnumber%N")
			else
				-- read the hostname, username and portnumber from the command line
				server_address := argv.item (1)
				name := argv.item(2)
				port := argv.item(3).to_integer
				
				-- create the users list	
				create user_list.make
			
				-- create a network datagram socket bound to its well
				-- known local address with port 'port'.
				create soc.make_bound (port)				

				-- create and launch a thread used for reading commands from the socket
				create receiver_thread.make_me (soc, user_list)
				receiver_thread.launch

				-- create and launch a thread used for receiving the list of all active clients and 
				-- sending information to register with the server
				create registration_thread.make_me (server_address, name, port, user_list)
				registration_thread.launch

				-- Create poller for multi-event polling
				create ps_write.make
				-- create a write command which is attached to the socket
				create writecomm.make (soc)
				-- set 'writecomm' to be called when write event is selected on its io_medium
				ps_write.put_write_command (writecomm)

				-- create the UI menu
				create uiMenu.make_me (ps_write, writecomm, user_list)
				-- start the 'user interface'
				uiMenu.whatToDoMenu

				-- close the socket and stop the threads execution
				soc.close
				receiver_thread.stop
				registration_thread.stop
			end
		end
		
end -- class INSTANT_MESSENGER
