indexing
	description: "[
	  "A client that registers itself with the server. In 
	  return, the server also sends him a list of all the registered
	  clients. This uses a TCP/IP connection throughout the entire
	  communication.";
	  
	  Authors: Keno Albrecht and Angela Nicoara
	]"

	date: "$Date: 2005/04/14 14:42:40 $";
	revision: "$Revision: 1.3 $"

class REGISTRATION_CLIENT

inherit

	NETWORK_CLIENT

create

	make_client

feature

	server_address: STRING
	name: STRING
	port: INTEGER
	user_list: LINKED_LIST[USER]
	
	REGISTER: INTEGER

	make_client (server_address_: STRING; name_: STRING; port_: INTEGER; user_list_: LINKED_LIST[USER]) is
	do
		server_address := server_address_
		name := name_
		port := port_
		user_list := user_list_

		make (5678, server_address)
	end

	write16(value: INTEGER) is
	-- Writes the lower 2 bytes of the given integer to the stream 
	-- (most significant byte first)
	local
		ch1: CHARACTER
		ch2: CHARACTER
	do
		ch1 := value.to_character
		ch2 := (value // 256).to_character
		
		-- write to stream
		in_out.put_character (ch2)
		in_out.put_character (ch1)		
	end

	read16: INTEGER is
	-- Read a 2-byte number from the stream (most significant first) 
	-- and returns it as integer. We read two bytes from the input 
	-- stream as b1 and b2. The resultis a four byte integer value 
	-- given as i1,i2,i3,i4. So we have to make sure that i3==b1 and i4==b2.
	local
		ch1: CHARACTER
		ch2: CHARACTER
		value: INTEGER
	do
		-- read ch1+ch2 from stream
		in_out.read_character
		ch1 := in_out.last_character
		in_out.read_character
		ch2 := in_out.last_character		
		
		value := ch1.code*256 + ch2.code

		Result := value
	end	

	registerUser is
 	 -- Sends the following information to register with the server 
 	 -- (number of bytes given in parenthesis):
	 -- type of message: REGISTER (1), own port (2), length of name (1),
	 -- own name (var <= 255).
	local
		i: INTEGER
	do
		REGISTER := 1
		
		in_out.put_character (REGISTER.to_character)
		write16(port)
		in_out.put_character (name.count.to_character) --OK before
		from i:=1 until i > name.count loop
			in_out.put_character (name.item(i))
			i := i+1
		end
	end

	receiveUserList is
	-- The server sends the list of all active clients (number of bytes given
	-- in parenthesis):
	-- type of message: USERLIST (1), number of users (2), list of users,
	-- where each user is defined as: length of name (1), name (var <= 255),
	-- ip (4), port (2). The ip is given with most significant byte first, ie.
	-- the address 129.132.130.158 is sent/received with 129 as first byte,
	-- 132 second...
	--
	-- Receive the user list and print it to the standard output. Give a
	-- hardcopy to your tutor.			
	local
		type: CHARACTER
		clientNumber: INTEGER
		i: INTEGER
		j: INTEGER
		length: INTEGER
		c: CHARACTER
		userName: STRING
		userPort: INTEGER		
		ip: STRING  --angy
		user: USER
	do
		user_list.wipe_out -- empties the list
		
		in_out.read_character
		type := in_out.last_character
		clientNumber := read16
		from i:=1 until i > clientNumber loop
			in_out.read_character
			-- read length of name
			length := in_out.last_character.code
			-- read name
			userName := ""
			from j:=1 until j > length loop
				in_out.read_character
				c := in_out.last_character
				userName.append_character (c)
				j := j+1
			end
	
			-- read IP
			ip := ""
			in_out.read_character
			ip.append_integer (in_out.last_character.code)  --angy
			ip.append_character ('.')  --angy
			in_out.read_character
			ip.append_integer (in_out.last_character.code)  --angy			
			ip.append_character ('.')  --angy
			in_out.read_character
			ip.append_integer (in_out.last_character.code)  --angy
			ip.append_character ('.')  --angy
			in_out.read_character
			ip.append_integer (in_out.last_character.code)  --angy
			
			-- read port
			userPort := read16

			create user.make_me (userName, ip, userPort)
			
			-- append the user to the list
			user_list.extend (user)

			i := i+1
		end
	end		

end